@@ -22,6 +22,7 @@
 #include <fcntl.h>
 
 #include <glib/gstdio.h>
+#include <locale.h>
 
 #include <libtracker-common/tracker-common.h>
 #include <libtracker-common/tracker-parser.h>
@@ -32,8 +33,6 @@
 #include "tracker-data-manager.h"
 #include "tracker-uuid.h"
 
-#define UNKNOWN_STATUS 0.5
-
 #define MAX_INTERFACES_PER_CPU        16
 #define MAX_INTERFACES                (MAX_INTERFACES_PER_CPU * g_get_num_processors ())
 
@@ -42,7 +41,7 @@
 
 #define TRACKER_VACUUM_CHECK_SIZE     ((goffset) 4 * 1024 * 1024 * 1024) /* 4GB */
 
-#define IN_USE_FILENAME               ".meta.isrunning"
+#define CORRUPTED_FILENAME            ".meta.corrupted"
 
 #define TOSTRING1(x) #x
 #define TOSTRING(x) TOSTRING1(x)
@@ -74,20 +73,15 @@ static TrackerDBDefinition db_base = {
 struct _TrackerDBManager {
 	GObject parent_instance;
 	TrackerDBDefinition db;
-	gboolean locations_initialized;
 	gchar *data_dir;
-	gchar *user_data_dir;
-	gchar *in_use_filename;
+	gchar *corrupted_filename;
 	GFile *cache_location;
 	gchar *shared_cache_key;
 	TrackerDBManagerFlags flags;
 	guint s_cache_size;
-	guint u_cache_size;
 	gboolean first_time;
 	TrackerDBVersion db_version;
 
-	gpointer vtab_data;
-
 	GWeakRef iface_data;
 
 	GAsyncQueue *interfaces;
@@ -116,16 +110,8 @@ tracker_db_manager_is_first_time (TrackerDBManager *db_manager)
 }
 
 TrackerDBManagerFlags
-tracker_db_manager_get_flags (TrackerDBManager *db_manager,
-                              guint            *select_cache_size,
-                              guint            *update_cache_size)
+tracker_db_manager_get_flags (TrackerDBManager *db_manager)
 {
-	if (select_cache_size)
-		*select_cache_size = db_manager->s_cache_size;
-
-	if (update_cache_size)
-		*update_cache_size = db_manager->u_cache_size;
-
 	return db_manager->flags;
 }
 
@@ -134,7 +120,7 @@ iface_set_params (TrackerDBInterface   *iface,
                   gboolean              readonly,
                   GError              **error)
 {
-	tracker_db_interface_execute_query (iface, NULL, "PRAGMA encoding = \"UTF-8\"");
+	tracker_db_interface_execute_query (iface, NULL, "PRAGMA encoding = 'UTF-8'");
 
 	if (readonly) {
 		tracker_db_interface_execute_query (iface, NULL, "PRAGMA temp_store = MEMORY;");
@@ -170,11 +156,11 @@ db_set_params (TrackerDBInterface   *iface,
 			         internal_error->message);
 			g_propagate_error (error, internal_error);
 		} else {
-			TrackerDBCursor *cursor;
+			TrackerSparqlCursor *cursor;
 
-			cursor = tracker_db_statement_start_cursor (stmt, NULL);
-			if (tracker_db_cursor_iter_next (cursor, NULL, NULL)) {
-				if (g_ascii_strcasecmp (tracker_db_cursor_get_string (cursor, 0, NULL), "WAL") != 0) {
+			cursor = TRACKER_SPARQL_CURSOR (tracker_db_statement_start_cursor (stmt, NULL));
+			if (tracker_sparql_cursor_next (cursor, NULL, NULL)) {
+				if (g_ascii_strcasecmp (tracker_sparql_cursor_get_string (cursor, 0, NULL), "WAL") != 0) {
 					g_set_error (error,
 					             TRACKER_DB_INTERFACE_ERROR,
 					             TRACKER_DB_OPEN_ERROR,
@@ -200,7 +186,7 @@ tracker_db_manager_get_metadata (TrackerDBManager   *db_manager,
 {
 	TrackerDBInterface *iface;
 	TrackerDBStatement *stmt;
-	TrackerDBCursor *cursor;
+	TrackerSparqlCursor *cursor;
 
 	iface = tracker_db_manager_get_writable_db_interface (db_manager);
 	stmt = tracker_db_interface_create_statement (iface, TRACKER_DB_STATEMENT_CACHE_TYPE_NONE,
@@ -210,15 +196,15 @@ tracker_db_manager_get_metadata (TrackerDBManager   *db_manager,
 		return FALSE;
 
 	tracker_db_statement_bind_text (stmt, 0, key);
-	cursor = tracker_db_statement_start_cursor (stmt, NULL);
+	cursor = TRACKER_SPARQL_CURSOR (tracker_db_statement_start_cursor (stmt, NULL));
 	g_object_unref (stmt);
 
-	if (!cursor || !tracker_db_cursor_iter_next (cursor, NULL, NULL)) {
+	if (!cursor || !tracker_sparql_cursor_next (cursor, NULL, NULL)) {
 		g_clear_object (&cursor);
 		return FALSE;
 	}
 
-	tracker_db_cursor_get_value (cursor, 0, value);
+	tracker_db_cursor_get_value (TRACKER_DB_CURSOR (cursor), 0, value);
 	g_object_unref (cursor);
 
 	return G_VALUE_TYPE (value) != G_TYPE_INVALID;
@@ -256,7 +242,7 @@ db_get_version (TrackerDBManager *db_manager)
 {
 	TrackerDBInterface *iface;
 	TrackerDBStatement *stmt;
-	TrackerDBCursor *cursor;
+	TrackerSparqlCursor *cursor;
 	TrackerDBVersion version;
 
 	iface = tracker_db_manager_get_writable_db_interface (db_manager);
@@ -265,15 +251,15 @@ db_get_version (TrackerDBManager *db_manager)
 	if (!stmt)
 		return TRACKER_DB_VERSION_UNKNOWN;
 
-	cursor = tracker_db_statement_start_cursor (stmt, NULL);
+	cursor = TRACKER_SPARQL_CURSOR (tracker_db_statement_start_cursor (stmt, NULL));
 	g_object_unref (stmt);
 
-	if (!cursor || !tracker_db_cursor_iter_next (cursor, NULL, NULL)) {
+	if (!cursor || !tracker_sparql_cursor_next (cursor, NULL, NULL)) {
 		g_clear_object (&cursor);
 		return TRACKER_DB_VERSION_UNKNOWN;
 	}
 
-	version = tracker_db_cursor_get_int (cursor, 0);
+	version = tracker_sparql_cursor_get_integer (cursor, 0);
 	g_object_unref (cursor);
 
 	return version;
@@ -334,11 +320,11 @@ tracker_db_manager_locale_changed (TrackerDBManager  *db_manager,
                                    GError           **error)
 {
 	gchar *db_locale;
-	gchar *current_locale;
+	const gchar *current_locale;
 	gboolean changed;
 
 	/* Get current collation locale */
-	current_locale = tracker_locale_get (TRACKER_LOCALE_COLLATE);
+	current_locale = setlocale (LC_COLLATE, NULL);
 
 	/* Get db locale */
 	db_locale = db_get_locale (db_manager);
@@ -360,7 +346,6 @@ tracker_db_manager_locale_changed (TrackerDBManager  *db_manager,
 	}
 
 	g_free (db_locale);
-	g_free (current_locale);
 
 	return changed;
 }
@@ -368,13 +353,12 @@ tracker_db_manager_locale_changed (TrackerDBManager  *db_manager,
 void
 tracker_db_manager_set_current_locale (TrackerDBManager *db_manager)
 {
-	gchar *current_locale;
+	const gchar *current_locale;
 
 	/* Get current collation locale */
-	current_locale = tracker_locale_get (TRACKER_LOCALE_COLLATE);
+	current_locale = setlocale (LC_COLLATE, NULL);
 	g_debug ("Saving DB locale as: '%s'", current_locale);
 	db_set_locale (db_manager, current_locale);
-	g_free (current_locale);
 }
 
 static void
@@ -387,11 +371,6 @@ tracker_db_manager_ensure_location (TrackerDBManager *db_manager,
 		return;
 	}
 
-	if (db_manager->locations_initialized) {
-		return;
-	}
-
-	db_manager->locations_initialized = TRUE;
 	db_manager->data_dir = g_file_get_path (cache_location);
 
 	db_manager->db = db_base;
@@ -442,57 +421,46 @@ tracker_db_manager_rollback_db_creation (TrackerDBManager *db_manager)
 	g_free (filename);
 }
 
-static gboolean
-db_check_integrity (TrackerDBManager *db_manager)
+gboolean
+tracker_db_manager_check_integrity (TrackerDBManager  *db_manager,
+                                    GError           **error)
 {
-	GError *internal_error = NULL;
 	TrackerDBStatement *stmt;
-	TrackerDBCursor *cursor = NULL;
+	TrackerSparqlCursor *cursor = NULL;
 
 	stmt = tracker_db_interface_create_statement (db_manager->db.iface, TRACKER_DB_STATEMENT_CACHE_TYPE_NONE,
-	                                              &internal_error,
+	                                              error,
 	                                              "PRAGMA integrity_check(1)");
-
-	if (!stmt) {
-		if (internal_error) {
-			g_message ("Corrupt database: failed to create integrity_check statement: %s", internal_error->message);
-		}
-
-		g_clear_error (&internal_error);
+	if (!stmt)
 		return FALSE;
-	}
 
-	cursor = tracker_db_statement_start_cursor (stmt, NULL);
+	cursor = TRACKER_SPARQL_CURSOR (tracker_db_statement_start_cursor (stmt, error));
 	g_object_unref (stmt);
 
-	if (cursor) {
-		if (tracker_db_cursor_iter_next (cursor, NULL, NULL)) {
-			const gchar *check_result;
-
-			check_result = tracker_db_cursor_get_string (cursor, 0, NULL);
-			if (g_strcmp0 (check_result, "ok") != 0) {
-				g_message ("Corrupt database: sqlite integrity check returned '%s'", check_result);
-				return FALSE;
-			}
-		}
-		g_object_unref (cursor);
-	}
-
-	/* ensure that database has been initialized by an earlier tracker-store start
-	   by checking whether Resource table exists */
-	stmt = tracker_db_interface_create_statement (db_manager->db.iface, TRACKER_DB_STATEMENT_CACHE_TYPE_NONE,
-	                                              &internal_error,
-	                                              "SELECT 1 FROM Resource");
-	if (!stmt) {
-		if (internal_error) {
-			g_message ("Corrupt database: failed to create resource check statement: %s", internal_error->message);
-		}
-
-		g_clear_error (&internal_error);
+	if (!cursor)
 		return FALSE;
+
+	if (tracker_sparql_cursor_next (cursor, NULL, NULL)) {
+		const gchar *check_result;
+
+		check_result = tracker_sparql_cursor_get_string (cursor, 0, NULL);
+		if (g_strcmp0 (check_result, "ok") != 0) {
+			GError *inner_error = NULL;
+
+			if (!g_file_set_contents (db_manager->corrupted_filename, "", -1, &inner_error))
+				g_warning ("Could not save " CORRUPTED_FILENAME ": %s", inner_error->message);
+
+			g_clear_error (&inner_error);
+			g_clear_object (&cursor);
+			g_set_error (error,
+			             TRACKER_DB_INTERFACE_ERROR,
+			             TRACKER_DB_CORRUPT,
+			             "Integrity check failed: %s", check_result);
+			return FALSE;
+		}
 	}
 
-	g_object_unref (stmt);
+	g_object_unref (cursor);
 
 	return TRUE;
 }
@@ -502,21 +470,15 @@ tracker_db_manager_new (TrackerDBManagerFlags   flags,
                         GFile                  *cache_location,
                         gboolean                shared_cache,
                         guint                   select_cache_size,
-                        guint                   update_cache_size,
-                        TrackerBusyCallback     busy_callback,
-                        gpointer                busy_user_data,
                         GObject                *iface_data,
-                        gpointer                vtab_data,
                         GError                **error)
 {
 	TrackerDBManager *db_manager;
-	int in_use_file;
 	TrackerDBInterface *resources_iface;
 	GError *internal_error = NULL;
 	gboolean need_to_create = FALSE;
 
 	db_manager = g_object_new (TRACKER_TYPE_DB_MANAGER, NULL);
-	db_manager->vtab_data = vtab_data;
 
 	/* Set default value for first_time */
 	db_manager->first_time = FALSE;
@@ -524,7 +486,6 @@ tracker_db_manager_new (TrackerDBManagerFlags   flags,
 	/* Set up locations */
 	db_manager->flags = flags;
 	db_manager->s_cache_size = select_cache_size;
-	db_manager->u_cache_size = update_cache_size;
 	db_manager->interfaces = g_async_queue_new_full (g_object_unref);
 
 	g_set_object (&db_manager->cache_location, cache_location);
@@ -532,9 +493,9 @@ tracker_db_manager_new (TrackerDBManagerFlags   flags,
 
 	if ((db_manager->flags & TRACKER_DB_MANAGER_IN_MEMORY) == 0) {
 		tracker_db_manager_ensure_location (db_manager, cache_location);
-		db_manager->in_use_filename = g_build_filename (db_manager->data_dir,
-		                                                IN_USE_FILENAME,
-		                                                NULL);
+		db_manager->corrupted_filename = g_build_filename (db_manager->data_dir,
+		                                                   CORRUPTED_FILENAME,
+		                                                   NULL);
 
 		/* Don't do need_reindex checks for readonly (direct-access) */
 		if ((flags & TRACKER_DB_MANAGER_READONLY) == 0) {
@@ -594,8 +555,6 @@ tracker_db_manager_new (TrackerDBManagerFlags   flags,
 		}
 	}
 
-	db_manager->locations_initialized = TRUE;
-
 	if ((flags & TRACKER_DB_MANAGER_READONLY) != 0) {
 		GValue value = G_VALUE_INIT;
 		TrackerDBManagerFlags fts_flags = 0;
@@ -620,8 +579,7 @@ tracker_db_manager_new (TrackerDBManagerFlags   flags,
 	if (need_to_create) {
 		db_manager->first_time = TRUE;
 
-		if ((db_manager->flags & TRACKER_DB_MANAGER_IN_MEMORY) == 0 &&
-		     !tracker_file_system_has_enough_space (db_manager->data_dir, TRACKER_DB_MIN_REQUIRED_SPACE, TRUE)) {
+		if (!tracker_db_manager_has_enough_space (db_manager)) {
 			g_set_error (error,
 			             TRACKER_DB_INTERFACE_ERROR,
 			             TRACKER_DB_OPEN_ERROR,
@@ -639,64 +597,6 @@ tracker_db_manager_new (TrackerDBManagerFlags   flags,
 		}
 
 		g_clear_object (&db_manager->db.iface);
-	} else {
-		TRACKER_NOTE (SQLITE, g_message ("Loading files for database %s...", db_manager->db.abs_filename));
-
-		if ((flags & TRACKER_DB_MANAGER_READONLY) == 0) {
-			/* Check that the database was closed cleanly and do a deeper integrity
-			 * check if it wasn't, raising an error if we detect corruption. */
-
-			if (g_file_test (db_manager->in_use_filename, G_FILE_TEST_EXISTS)) {
-				gsize size = 0;
-				struct stat st;
-
-				TRACKER_NOTE (SQLITE, g_message ("Didn't shut down cleanly last time, doing integrity checks"));
-
-				if (g_stat (db_manager->db.abs_filename, &st) == 0) {
-					size = st.st_size;
-				}
-
-				/* Size is 1 when using echo > file.db, none of our databases
-				 * are only one byte in size even initually. */
-				if (size <= 1) {
-					g_debug ("Database is corrupt: size is 1 byte or less.");
-					return FALSE;
-				}
-
-				db_manager->db.iface = tracker_db_manager_create_db_interface (db_manager, FALSE, &internal_error);
-
-				if (internal_error) {
-					/* If this already doesn't succeed, then surely the file is
-					 * corrupt. No need to check for integrity anymore. */
-					g_propagate_error (error, internal_error);
-					g_object_unref (db_manager);
-					return NULL;
-				}
-
-				busy_callback ("Integrity checking", 0, busy_user_data);
-
-				if (db_check_integrity (db_manager) == FALSE) {
-					g_set_error (error,
-					             TRACKER_DB_INTERFACE_ERROR,
-					             TRACKER_DB_OPEN_ERROR,
-					             "Corrupt db file");
-					g_object_unref (db_manager);
-					return NULL;
-				}
-			}
-		}
-	}
-
-	if ((flags & (TRACKER_DB_MANAGER_READONLY | TRACKER_DB_MANAGER_IN_MEMORY)) == 0) {
-		/* do not create in-use file for read-only mode (direct access) */
-		in_use_file = g_open (db_manager->in_use_filename,
-			              O_WRONLY | O_APPEND | O_CREAT | O_SYNC,
-			              S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
-
-		if (in_use_file >= 0) {
-		        fsync (in_use_file);
-		        close (in_use_file);
-		}
 	}
 
 	resources_iface = tracker_db_manager_create_db_interface (db_manager,
@@ -719,6 +619,8 @@ tracker_db_manager_finalize (GObject *object)
 	TrackerDBManager *db_manager = TRACKER_DB_MANAGER (object);
 	gboolean readonly = (db_manager->flags & TRACKER_DB_MANAGER_READONLY) != 0;
 
+	tracker_db_manager_release_memory (db_manager);
+
 	g_async_queue_unref (db_manager->interfaces);
 	g_free (db_manager->db.abs_filename);
 
@@ -732,13 +634,7 @@ tracker_db_manager_finalize (GObject *object)
 
 	g_free (db_manager->data_dir);
 
-	if (db_manager->in_use_filename && !readonly) {
-		/* do not delete in-use file for read-only mode (direct access) */
-		if (g_unlink (db_manager->in_use_filename) < 0)
-			g_warning ("Could not delete '" IN_USE_FILENAME "': %m");
-	}
-
-	g_free (db_manager->in_use_filename);
+	g_free (db_manager->corrupted_filename);
 	g_free (db_manager->shared_cache_key);
 	g_clear_object (&db_manager->cache_location);
 
@@ -753,6 +649,7 @@ tracker_db_manager_create_db_interface (TrackerDBManager  *db_manager,
 	TrackerDBInterface *connection;
 	GError *internal_error = NULL;
 	TrackerDBInterfaceFlags flags = 0;
+	GObject *user_data;
 
 	if (readonly)
 		flags |= TRACKER_DB_INTERFACE_READONLY;
@@ -770,12 +667,11 @@ tracker_db_manager_create_db_interface (TrackerDBManager  *db_manager,
 		return NULL;
 	}
 
-	tracker_db_interface_set_user_data (connection,
-	                                    g_weak_ref_get (&db_manager->iface_data),
-	                                    g_object_unref);
+	user_data = g_weak_ref_get (&db_manager->iface_data);
+	tracker_db_interface_set_user_data (connection, user_data);
+	g_object_unref (user_data);
 
-	if (db_manager->vtab_data)
-		tracker_db_interface_init_vtabs (connection, db_manager->vtab_data);
+	tracker_db_interface_init_vtabs (connection);
 
 	iface_set_params (connection,
 	                  readonly,
@@ -792,16 +688,15 @@ tracker_db_manager_create_db_interface (TrackerDBManager  *db_manager,
 		return NULL;
 	}
 
+	if (!tracker_db_interface_sqlite_fts_init (connection,
+	                                           db_manager->flags,
+	                                           error))
+		return FALSE;
+
 	tracker_db_interface_set_max_stmt_cache_size (connection,
 	                                              TRACKER_DB_STATEMENT_CACHE_TYPE_SELECT,
 	                                              db_manager->s_cache_size);
 
-	if (!readonly) {
-		tracker_db_interface_set_max_stmt_cache_size (connection,
-		                                              TRACKER_DB_STATEMENT_CACHE_TYPE_UPDATE,
-		                                              db_manager->u_cache_size);
-	}
-
 	return connection;
 }
 
@@ -1115,10 +1010,20 @@ tracker_db_manager_release_memory (TrackerDBManager *db_manager)
 		if (!iface)
 			break;
 
-		if (tracker_db_interface_get_is_used (iface))
+		if (tracker_db_interface_get_is_used (iface)) {
 			g_async_queue_push_unlocked (db_manager->interfaces, iface);
-		else
+		} else {
+			if (tracker_db_interface_found_corruption (iface)) {
+				GError *error = NULL;
+
+				if (!g_file_set_contents (db_manager->corrupted_filename, "", -1, &error))
+					g_warning ("Could not save " CORRUPTED_FILENAME ": %s", error->message);
+
+				g_clear_error (&error);
+			}
+
 			g_object_unref (iface);
+		}
 	}
 
 	if (g_async_queue_length_unlocked (db_manager->interfaces) < len) {
@@ -1145,3 +1050,16 @@ tracker_db_manager_get_version (TrackerDBManager *db_manager)
 {
 	return db_manager->db_version;
 }
+
+gboolean
+tracker_db_manager_needs_repair (TrackerDBManager *db_manager)
+{
+	if (g_file_test (db_manager->corrupted_filename, G_FILE_TEST_EXISTS)) {
+		if (g_unlink (db_manager->corrupted_filename) < 0)
+			g_warning ("Could not delete " CORRUPTED_FILENAME ": %m");
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
