@@ -35,6 +35,7 @@ struct _TrackerQuad
 	gchar *predicate;
 	gchar *object;
 	gchar *graph;
+	gchar *object_langtag;
 	TrackerSparqlValueType subject_type;
 	TrackerSparqlValueType object_type;
 };
@@ -42,7 +43,7 @@ struct _TrackerQuad
 struct _TrackerSerializerTrig
 {
 	TrackerSerializer parent_instance;
-	TrackerQuad *last_quad;
+	TrackerQuad last_quad;
 	GString *data;
 	guint stream_closed : 1;
 	guint cursor_started : 1;
@@ -63,23 +64,23 @@ typedef enum
 	TRACKER_QUAD_BREAK_OBJECT,
 } TrackerQuadBreak;
 
-static TrackerQuad *
-tracker_quad_new_from_cursor (TrackerSparqlCursor *cursor)
+static void
+tracker_quad_init_from_cursor (TrackerQuad         *quad,
+                               TrackerSparqlCursor *cursor)
 {
-	TrackerQuad *quad;
+	const gchar *langtag;
 
-	if (tracker_sparql_cursor_get_n_columns (cursor) < 3)
-		return NULL;
-
-	quad = g_new0 (TrackerQuad, 1);
 	quad->subject_type = tracker_sparql_cursor_get_value_type (cursor, 0);
 	quad->object_type = tracker_sparql_cursor_get_value_type (cursor, 2);
 	quad->subject = g_strdup (tracker_sparql_cursor_get_string (cursor, 0, NULL));
 	quad->predicate = g_strdup (tracker_sparql_cursor_get_string (cursor, 1, NULL));
-	quad->object = g_strdup (tracker_sparql_cursor_get_string (cursor, 2, NULL));
+	quad->object = g_strdup (tracker_sparql_cursor_get_langstring (cursor, 2, &langtag, NULL));
+	quad->object_langtag = g_strdup (langtag);
 
 	if (tracker_sparql_cursor_get_n_columns (cursor) >= 4)
 		quad->graph = g_strdup (tracker_sparql_cursor_get_string (cursor, 3, NULL));
+	else
+		quad->graph = NULL;
 
 	if (quad->subject_type == TRACKER_SPARQL_VALUE_TYPE_STRING) {
 		if (g_str_has_prefix (quad->subject, "urn:bnode:")) {
@@ -94,25 +95,23 @@ tracker_quad_new_from_cursor (TrackerSparqlCursor *cursor)
 			quad->object_type = TRACKER_SPARQL_VALUE_TYPE_BLANK_NODE;
 		}
 	}
-
-	return quad;
 }
 
 static void
-tracker_quad_free (TrackerQuad *quad)
+tracker_quad_clear (TrackerQuad *quad)
 {
-	g_free (quad->subject);
-	g_free (quad->predicate);
-	g_free (quad->object);
-	g_free (quad->graph);
-	g_free (quad);
+	g_clear_pointer (&quad->subject, g_free);
+	g_clear_pointer (&quad->predicate, g_free);
+	g_clear_pointer (&quad->object, g_free);
+	g_clear_pointer (&quad->graph, g_free);
+	g_clear_pointer (&quad->object_langtag, g_free);
 }
 
 static TrackerQuadBreak
 tracker_quad_get_break (TrackerQuad *last,
                         TrackerQuad *cur)
 {
-	if (!last)
+	if (!last->subject)
 		return TRACKER_QUAD_BREAK_NONE;
 
 	if (g_strcmp0 (last->graph, cur->graph) != 0)
@@ -148,9 +147,11 @@ print_value (GString                 *str,
 		shortname = tracker_namespace_manager_compress_uri (namespaces, value);
 
 		if (shortname) {
-			g_string_append_printf (str, "%s", shortname);
+			g_string_append (str, shortname);
 		} else {
-			g_string_append_printf (str, "<%s>", value);
+			g_string_append_c (str, '<');
+			g_string_append (str, value);
+			g_string_append_c (str, '>');
 		}
 
 		g_free (shortname);
@@ -159,9 +160,14 @@ print_value (GString                 *str,
 	case TRACKER_SPARQL_VALUE_TYPE_BLANK_NODE: {
 		gchar *bnode_label;
 
-		bnode_label = g_strdelimit (g_strdup (value), ":", '_');
-		g_string_append_printf (str, "_:%s", bnode_label);
-		g_free (bnode_label);
+		if (g_str_has_prefix (value, "_:")) {
+			g_string_append (str, value);
+		} else {
+			bnode_label = g_strdelimit (g_strdup (value), ":", '_');
+			g_string_append (str, "_:");
+			g_string_append (str, bnode_label);
+			g_free (bnode_label);
+		}
 		break;
 	}
 	case TRACKER_SPARQL_VALUE_TYPE_STRING:
@@ -169,8 +175,9 @@ print_value (GString                 *str,
 		gchar *escaped;
 
 		escaped = tracker_sparql_escape_string (value);
-		g_string_append_printf (str, "\"%s\"",
-					escaped);
+		g_string_append_c (str, '"');
+		g_string_append (str, escaped);
+		g_string_append_c (str, '"');
 		g_free (escaped);
 		break;
 	}
@@ -190,14 +197,14 @@ print_value (GString                 *str,
 
 static gboolean
 serialize_up_to_size (TrackerSerializerTrig  *serializer_trig,
-		      gsize                   size,
-		      GCancellable           *cancellable,
-		      GError                **error)
+                      gsize                   size,
+                      GCancellable           *cancellable,
+                      GError                **error)
 {
 	TrackerSparqlCursor *cursor;
 	TrackerNamespaceManager *namespaces;
 	GError *inner_error = NULL;
-	TrackerQuad *cur;
+	TrackerQuad cur;
 
 	if (!serializer_trig->data)
 		serializer_trig->data = g_string_new (NULL);
@@ -210,7 +217,8 @@ serialize_up_to_size (TrackerSerializerTrig  *serializer_trig,
 
 		str = tracker_namespace_manager_print_turtle (namespaces);
 
-		g_string_append_printf (serializer_trig->data, "%s\n", str);
+		g_string_append (serializer_trig->data, str);
+		g_string_append_c (serializer_trig->data, '\n');
 		g_free (str);
 		serializer_trig->head_printed = TRUE;
 	}
@@ -231,9 +239,9 @@ serialize_up_to_size (TrackerSerializerTrig  *serializer_trig,
 			serializer_trig->cursor_started = TRUE;
 		}
 
-		cur = tracker_quad_new_from_cursor (cursor);
+		tracker_quad_init_from_cursor (&cur, cursor);
 
-		if (!cur) {
+		if (!cur.subject && cur.predicate && cur.object) {
 			g_set_error (error,
 			             TRACKER_SPARQL_ERROR,
 			             TRACKER_SPARQL_ERROR_INTERNAL,
@@ -241,15 +249,15 @@ serialize_up_to_size (TrackerSerializerTrig  *serializer_trig,
 			return FALSE;
 		}
 
-		br = tracker_quad_get_break (serializer_trig->last_quad, cur);
+		br = tracker_quad_get_break (&serializer_trig->last_quad, &cur);
 
 		if (br <= TRACKER_QUAD_BREAK_GRAPH) {
 			if (br == TRACKER_QUAD_BREAK_GRAPH)
 				g_string_append (serializer_trig->data, " .\n}\n\n");
 
-			if (cur->graph) {
+			if (cur.graph) {
 				g_string_append (serializer_trig->data, "GRAPH ");
-				print_value (serializer_trig->data, cur->graph,
+				print_value (serializer_trig->data, cur.graph,
 				             TRACKER_SPARQL_VALUE_TYPE_URI, namespaces);
 				g_string_append_c (serializer_trig->data, ' ');
 			}
@@ -260,7 +268,7 @@ serialize_up_to_size (TrackerSerializerTrig  *serializer_trig,
 		if (br <= TRACKER_QUAD_BREAK_SUBJECT) {
 			if (br == TRACKER_QUAD_BREAK_SUBJECT)
 				g_string_append (serializer_trig->data, " .\n\n  ");
-			print_value (serializer_trig->data, cur->subject, cur->subject_type, namespaces);
+			print_value (serializer_trig->data, cur.subject, cur.subject_type, namespaces);
 		}
 
 		if (br <= TRACKER_QUAD_BREAK_PREDICATE) {
@@ -269,7 +277,7 @@ serialize_up_to_size (TrackerSerializerTrig  *serializer_trig,
 			else
 				g_string_append_c (serializer_trig->data, ' ');
 
-			print_value (serializer_trig->data, cur->predicate,
+			print_value (serializer_trig->data, cur.predicate,
 			             TRACKER_SPARQL_VALUE_TYPE_URI, namespaces);
 		}
 
@@ -278,18 +286,25 @@ serialize_up_to_size (TrackerSerializerTrig  *serializer_trig,
 				g_string_append (serializer_trig->data, ",");
 
 			g_string_append_c (serializer_trig->data, ' ');
-			print_value (serializer_trig->data, cur->object, cur->object_type, namespaces);
+			print_value (serializer_trig->data, cur.object, cur.object_type, namespaces);
+
+			if (cur.object_langtag) {
+				g_string_append_c (serializer_trig->data, '@');
+				g_string_append (serializer_trig->data, cur.object_langtag);
+			}
 		}
 
 		serializer_trig->has_quads = TRUE;
-		g_clear_pointer (&serializer_trig->last_quad, tracker_quad_free);
-		serializer_trig->last_quad = cur;
+		tracker_quad_clear (&serializer_trig->last_quad);
+		memcpy (&serializer_trig->last_quad, &cur, sizeof (TrackerQuad));
 	}
 
 	/* Close the last quad */
 	if (serializer_trig->cursor_finished &&
-	    serializer_trig->has_quads)
+	    serializer_trig->has_quads) {
 		g_string_append (serializer_trig->data, " .\n}\n");
+		serializer_trig->has_quads = FALSE;
+	}
 
 	return TRUE;
 }
@@ -332,7 +347,7 @@ tracker_serializer_trig_close (GInputStream  *istream,
 {
 	TrackerSerializerTrig *serializer_trig = TRACKER_SERIALIZER_TRIG (istream);
 
-	g_clear_pointer (&serializer_trig->last_quad, tracker_quad_free);
+	tracker_quad_clear (&serializer_trig->last_quad);
 
 	if (serializer_trig->data) {
 		g_string_free (serializer_trig->data, TRUE);
